@@ -86,6 +86,14 @@ def test_content_length_setter():
     assert 234 == resp.content_length
 
 
+def test_content_length_setter_with_enable_chunked_encoding():
+    resp = StreamResponse()
+
+    resp.enable_chunked_encoding()
+    with pytest.raises(RuntimeError):
+        resp.content_length = 234
+
+
 def test_drop_content_length_header_on_setting_len_to_None():
     resp = StreamResponse()
 
@@ -223,13 +231,22 @@ def test_chunked_encoding():
     assert msg.chunked
 
 
+def test_enable_chunked_encoding_with_content_length():
+    resp = StreamResponse()
+
+    resp.content_length = 234
+    with pytest.raises(RuntimeError):
+        resp.enable_chunked_encoding()
+
+
 @asyncio.coroutine
 def test_chunk_size():
     req = make_request('GET', '/', payload_writer=mock.Mock())
     resp = StreamResponse()
     assert not resp.chunked
 
-    resp.enable_chunked_encoding(chunk_size=8192)
+    with pytest.warns(DeprecationWarning):
+        resp.enable_chunked_encoding(chunk_size=8192)
     assert resp.chunked
 
     msg = yield from resp.prepare(req)
@@ -889,6 +906,20 @@ def test_send_set_cookie_header(buf, writer):
                     'Server: .+\r\n\r\n', txt)
 
 
+@asyncio.coroutine
+def test_consecutive_write_eof():
+    req = make_request('GET', '/')
+    data = b'data'
+    resp = Response(body=data)
+
+    yield from resp.prepare(req)
+    with mock.patch('aiohttp.web.StreamResponse.write_eof') as super_write_eof:
+        yield from resp.write_eof()
+        resp._eof_sent = True
+        yield from resp.write_eof()
+        super_write_eof.assert_called_once_with(data)
+
+
 def test_set_text_with_content_type():
     resp = Response()
     resp.content_type = "text/html"
